#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdlib.h>
#include <stdio.h>

char comm;
void USART_Init()
{
	/* setare baud rate */
	UBRR0H = 0x00;  //9600
	UBRR0L = 103; 

	/* enable receiver pentru bluetooth */
	UCSR0B = (1<<RXEN0);
	
	/* setare interrupt pe RX */
	UCSR0B |= (1<<RXCIE0);
}


ISR(USART0_RX_vect)
{
	comm = UDR0;
	switch(comm) {

	/* miscare inainte */
	case 'W':
		PORTD ^= (1 << PD7);
		
		PORTA |= (1 << PA3);
		PORTA &= ~(1 << PA2);
		
		break;
	
	/* miscare inapoi */
	case 'S':
		PORTD ^= (1 << PD7);
		
		PORTA |= (1 << PA2);
		PORTA &= ~(1 << PA3);
		
		break;
	
	/* oprire tractiune */
	case 'T':
		PORTD &= ~(1 << PB7);
		
		PORTA &= ~(1 << PA3);
		PORTA &= ~(1 << PA2);
		
		break;
		
	/* viteza rapida */
	case '1':
		PORTD ^= (1 << PD7);
		
		OCR0A = 260;
		
		break;
		
	/* viteza medie */
	case '2':		
		PORTD ^= (1 << PD7);

		OCR0A = 180;
		
		break;
		
	/* viteza mica */
	case '3':
		PORTD ^= (1 << PD7);
		
		OCR0A = 100;
	
		break;

	/* viraj stanga */
	case 'A':
		PORTD ^= (1 << PD7);
		
		PORTA &= ~(1 << PA0);
		PORTA |= (1 << PA1);
		
		break;
		
	/* viraj dreapta */
	case 'D':
		PORTD ^= (1 << PD7);
		
		PORTA &= ~(1 << PA1);
		PORTA |= (1 << PA0);
		
		break;
	
	/* resetare viraj/drept inainte */
	case 'Y':
		PORTD &= ~(1 << PB7);
		
		PORTA &= ~(1 << PA0);
		PORTA &= ~(1 << PA1);
		
		break;
	}
	
	_delay_ms(100);
}

void timer_init(void)
{
	/* Fast PWM, non-inverted, TOP OXFF and set on bottom */
	TCCR0A = (3 << WGM00) | (1 << COM0A1);
    TCCR0B = (1 << CS00) | (1 << CS02);
	
	/* viteza de start */
    OCR0A = 50;
}

void direction_init(void)
{
	/* init motor viraj | dreapta */
	DDRA |= (1 << PA0);
	PORTA &= ~(1 << PA0);
	
	/* init motor viraj | stanga */
	DDRA |= (1 << PA1);
	PORTA &= ~(1 << PA1);
	
	/* enable motor viraj | fara PWM */
	DDRA |= (1 << PA2);
	PORTA |= (1 << PA2);
}

void move_init(void)
{
	/* enable motor tractiune | cu PWM */
	DDRB |= (1 << PB3);
	
	/* init motor tractiune | inainte */
	DDRA |= (1 << PA3);
	PORTA &= ~(1 << PA3);
	
	/* init motor tractiune | inapoi */
	DDRA |= (1 << PA2);
	PORTA &= ~(1 << PA2);
}

int main(void)
{
	/* se foloseste un LED pentru a verifica daca comenzile sunt trimise prin bluetooth */
	DDRD |= (1 << PD7);
	
	direction_init();
	move_init();
	
	timer_init();

	USART_Init();
	
	/* enable pentru interrupt */
	sei();
	
	while(1);
}